#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <stddef.h>	// offsetof

#pragma comment(lib, "comctl32.lib")
#pragma comment(lib, "comdlg32.lib")

#pragma intrinsic(memcmp)

#define IDM_EXIT 6000
#define IDM_NEW 6001
#define IDM_OPEN 6002

#define IDC_STATUS 4000
#define IDC_TOOLBAR 4001
#define IDC_TREE 4002
#define IDC_LIST 4003
#define IDC_HVIEW 4004
#define IDC_HVIEW2 4005

#define INFOLIBMH 128

#ifndef NELEMS
#define NELEMS(a)  (sizeof(a) / sizeof(a[0]))
#endif

static LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
static BOOL OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct);
static void OnDestroy(HWND hwnd);
static void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void OnSize(HWND hwnd, UINT state, int cx, int cy);
static void OnNotify(HWND hwnd, int idCtrl, NMHDR* pNMHdr);
static void OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);
static void OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags);
static void OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags);
static void LVCellClick(LPARAM lParam);

static int DoOpenFileName(HWND hWnd);
static int DoOpenFile(HWND hWnd);
static int DoCloseFile(HWND hWnd);
static void DoTreeViewSelChng(void);
static LRESULT ListViewNotify(HWND hWnd, NMHDR* pNMHdr);
static LRESULT ListViewNotify2(HWND hWnd, NMHDR* pNMHdr);

static BOOL TreeViewClear(HWND hTree);
static HTREEITEM TreeViewInsert(HTREEITEM hParent, HTREEITEM hInsertAfter, const char *szText, LPARAM lParam);

TCHAR szAppName[] = TEXT("MSPDBView");
TCHAR szFrameClass[] = TEXT("cWSDIFrame");
HWND g_hFrame, g_hStatus, g_hToolbar;
HANDLE g_hInst;
TCHAR *g_szCmdLine;
HMENU g_hMenu, g_hMenuT;
HWND g_hWndTV, g_hWndLVC;
HWND g_hWndLV, g_hWndHV, g_hWndHV2;
HFONT g_hFont1,g_hFont2;
HTREEITEM g_hTVRoot;

static BOOL bDragX = FALSE; 	//for splitter
static UINT uDragX = 220;		//for splitter
HWND g_hWndFocus;

char g_szFileName[MAX_PATH];
char g_szFileTitle[MAX_PATH];
BYTE *g_pData, *g_pHex;
DWORD g_nFileSize, g_nHexMax;
// MSF
DWORD g_nBlockSize;
DWORD *g_pStreamSizes;	// pointer of array of Stream Sizes
DWORD *g_pStreamBlocks;	// pointer of array of Stream Blocks
// program additions
DWORD *g_pStrmBlockCnt1;	// pointer of array of Stream block count
DWORD *g_pStrmBlockNum1;	// pointer of array of Stream 1. block values
// hexview 2
BYTE *g_pHex2;
DWORD *g_pStreamBlocks2;	// pointer for Stream Blocks view
DWORD g_nHexMax2, g_nBlkIdx;	// current block index

#ifndef _TLPESHOWINFO_H
typedef struct _TNODE {
//	LPARAM Pos:16;	// WORD 0-65535
	LPARAM Pos:16+8;	// 0-8388608
	LPARAM Info:8;	// BYTE 0-255  > 250 type
}TNODE, *PTNODE;

typedef struct _FINFO {
	DWORD_PTR Ptr;	// UINT_PTR pointer to data
	DWORD_PTR Ptr2;	// additional pointer for handler
	DWORD Size;	// sizeof data
	DWORD Ofs;	/// Offset in file
	DWORD Info;	// handler info
} FINFO, *PFINFO;
#endif

PFINFO g_pFInfo;

DWORD g_fOpt;

typedef struct DbiStreamHeader DbiStreamHeader;
DbiStreamHeader *g_pDbiHdr;

#define MEMSIZE 10000

void Test(void);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	WNDCLASSEX wcx;
	MSG msg;

	wcx.cbSize = sizeof(WNDCLASSEX);
	//GetClassInfoEx(hInstance, MAKEINTRESOURCE(32770), &wcx);
	wcx.style = CS_HREDRAW | CS_VREDRAW;
	wcx.lpfnWndProc = (WNDPROC)WndProc;
	wcx.cbClsExtra = 0;
	wcx.cbWndExtra = 0;
	wcx.hInstance = hInstance;
	wcx.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	//wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcx.hCursor		= LoadCursor(NULL, IDC_SIZEWE); //for splitter
	//wcx.hbrBackground= (HBRUSH)COLOR_BACKGROUND+1;
	//wcx.hbrBackground = (HBRUSH)COLOR_WINDOW;
	wcx.hbrBackground = (HBRUSH)COLOR_APPWORKSPACE + 1;
	wcx.lpszMenuName = NULL;
	wcx.lpszClassName = szFrameClass;
	wcx.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

	if (!RegisterClassEx(&wcx))
		return 0;
	g_hInst = hInstance;
	g_szCmdLine = lpCmdLine;

	g_hFrame = CreateWindowEx(0, szFrameClass, szAppName,
				 WS_OVERLAPPEDWINDOW,
				 CW_USEDEFAULT, CW_USEDEFAULT,
				// CW_USEDEFAULT, CW_USEDEFAULT,
				 800, 500,
				 NULL, NULL, g_hInst, NULL);
	if (!g_hFrame)
		return 0;
	ShowWindow(g_hFrame, nCmdShow);
	//UpdateWindow(g_hFrame);

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}

HWND MakeToolbar(HWND hWnd, UINT wID)
{
	TBBUTTON tbb[] = {
		//{STD_, IDM_, TBSTATE_ENABLED, TBSTYLE_BUTTON, {0}, 0L, 0},
		{0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, {0}, 0L, 0},
		//{STD_FILENEW, IDM_NEW, TBSTATE_ENABLED, TBSTYLE_BUTTON, {0}, 0L, 0},
		{STD_FILEOPEN, IDM_OPEN, TBSTATE_ENABLED, TBSTYLE_BUTTON, {0}, 0L, 0},
	};

	HWND hToolbar = CreateToolbarEx(hWnd,
		WS_CHILD | WS_VISIBLE | TBSTYLE_TOOLTIPS | TBSTYLE_FLAT, wID, 0, HINST_COMMCTRL, IDB_STD_SMALL_COLOR,
		tbb, NELEMS(tbb), 100, 0, 30, 30, sizeof(TBBUTTON));
	return hToolbar;
}

HWND MakeStatusbar(HWND hWnd, UINT wID)
{
	HWND hStatus = CreateStatusWindow(WS_CHILD | WS_VISIBLE | SBARS_SIZEGRIP,
		NULL, hWnd, wID);
	return hStatus;
}

HMENU MakeMenubar(HWND hWnd)
{
	HMENU hMenu, hMenuT;

	hMenu = CreateMenu();
	hMenuT = CreatePopupMenu();
	AppendMenu(hMenuT, MF_ENABLED | MF_STRING, IDM_OPEN, TEXT("&Open"));
	AppendMenu(hMenuT, MF_ENABLED | MF_STRING, IDM_EXIT, TEXT("&Exit"));
	AppendMenu(hMenu, MF_ENABLED | MF_POPUP, (UINT_PTR)hMenuT, TEXT("&File"));
	SetMenu(hWnd, hMenu);
	return hMenu;
}

HWND MakeTreeView(HWND hWnd, UINT wId)
{
	HWND hWndTV = CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, NULL,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP |
			TVS_HASLINES | TVS_LINESATROOT |
			TVS_HASBUTTONS
			,0, 0, 0, 0,
			hWnd, (HMENU)wId, g_hInst, NULL);
	return hWndTV;
}

HWND MakeListView(HWND hWnd, UINT wID)
{
	HWND hWndLV;
	LV_COLUMN lvc[] = {
		// mask, fmt, cx, pszText, cchTextMax, iSubItem
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 80, TEXT("pFile"), 0, 0},
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_RIGHT, 80, TEXT("Data"), 0, 0},
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 200, TEXT("Description"), 0, 0},
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 250, TEXT("Value"), 0, 0},
	};
	int iC;

	hWndLV = CreateWindowEx(WS_EX_CLIENTEDGE, WC_LISTVIEW, NULL,
		WS_CHILD | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_SHOWSELALWAYS
		,0, 0, 0, 0,
		hWnd, (HMENU)(UINT_PTR)wID, g_hInst, NULL);
	SendMessage(hWndLV,LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES, LVS_EX_GRIDLINES);
	SendMessage(hWndLV, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);
	SendMessage(hWndLV, LVM_SETBKCOLOR, 0, CLR_NONE);
	for (iC=0; iC<NELEMS(lvc); iC++)
		ListView_InsertColumn(hWndLV, iC, &lvc[iC]);
	//SendMessage(hWndLV, WM_SETFONT, (UINT)GetStockObject(ANSI_FIXED_FONT), 0L);
	SendMessage(hWndLV,LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES, LVS_EX_GRIDLINES);
	return hWndLV;
}

int CALLBACK EnumFontFamProc(const LOGFONTA *lpelf, const TEXTMETRIC *lpntm,
  DWORD FontType, LPARAM lParam)
{
	return 1;
}

HWND MakeListViewHex(HWND hWnd, UINT wID)
{
	HWND hWndLV;
	LV_COLUMN lvc[] = {
		// mask, fmt, cx, pszText, cchTextMax, iSubItem
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 70, TEXT("pFile  "), 50, 0},	// one less as header is bold
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 400, TEXT("00 01 02 03 04 05 06 07  08 09 0A 0B 0C 0D 0E FF"), 50, 0},
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 140, TEXT("Value           "), 50, 0},
	};
	int iC;
	hWndLV = CreateWindowEx(WS_EX_CLIENTEDGE, WC_LISTVIEW, NULL,
		 WS_CHILD | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_OWNERDATA,
		 0, 0, 0, 0, hWnd, (HMENU)(UINT_PTR)wID, g_hInst, NULL);
	SendMessage(hWndLV, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);
	SendMessage(hWndLV, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES, LVS_EX_GRIDLINES);
	SendMessage(hWndLV, LVM_SETBKCOLOR, 0, CLR_NONE);
	for (iC = 0; iC < NELEMS(lvc); iC++)
		ListView_InsertColumn(hWndLV, iC, &lvc[iC]);
	//SendMessage(hWndLV, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), 0L);
	//SendMessage(hWndHV, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE_USEHEADER);
	HDC hDC = GetDC(hWnd);
	int iFont = EnumFontFamilies(hDC, "Consolas", EnumFontFamProc, 0);
	if (iFont) {
		LOGFONT lf;
		memset(&lf, 0, sizeof(LOGFONT));
		lf.lfPitchAndFamily = FIXED_PITCH;
		lf.lfHeight = 14;
		lstrcpyn(lf.lfFaceName, "Consolas", 32);
		g_hFont2 = CreateFontIndirect(&lf);
		SendMessage(hWndLV, WM_SETFONT, (WPARAM)g_hFont2, 0L);
	} else
		SendMessage(hWndLV, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), 0L);
	return hWndLV;
}

HWND MakeListViewHex2(HWND hWnd, UINT wID)
{
	HWND hWndLV;
	LV_COLUMN lvc[] = {
		// mask, fmt, cx, pszText, cchTextMax, iSubItem
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 70, TEXT("pFile  "), 50, 0},	// one less as header is bold
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 400, TEXT("00 01 02 03 04 05 06 07  08 09 0A 0B 0C 0D 0E FF"), 50, 0},
		{LVCF_TEXT | LVCF_FMT | LVCF_WIDTH, LVCFMT_LEFT, 140, TEXT("Value           "), 50, 0},
	};
	int iC;
	hWndLV = CreateWindowEx(WS_EX_CLIENTEDGE, WC_LISTVIEW, NULL,
		 WS_CHILD | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_OWNERDATA,
		 0, 0, 0, 0, hWnd, (HMENU)(UINT_PTR)wID, g_hInst, NULL);
	SendMessage(hWndLV, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);
	SendMessage(hWndLV, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES, LVS_EX_GRIDLINES);
	SendMessage(hWndLV, LVM_SETBKCOLOR, 0, CLR_NONE);
	for (iC = 0; iC < NELEMS(lvc); iC++)
		ListView_InsertColumn(hWndLV, iC, &lvc[iC]);
	//SendMessage(hWndLV, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), 0L);
	//SendMessage(hWndHV, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE_USEHEADER);
	if (g_hFont2) {
		SendMessage(hWndLV, WM_SETFONT, (WPARAM)g_hFont2, 0L);
	} else
		SendMessage(hWndLV, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), 0L);
	return hWndLV;
}

static LRESULT CALLBACK WndProc(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
	switch (wMsg)
	{
		case WM_COMMAND:
			return OnCommand((hwnd), (int)(LOWORD(wParam)), (HWND) (lParam), (UINT)HIWORD(wParam)), 0;
		case WM_SIZE:
			return OnSize((hwnd), (UINT) (wParam), (int)(short)LOWORD(lParam), (int)(short)HIWORD(lParam)), 0;
		case WM_NOTIFY:
			return (OnNotify(hwnd,(int)(wParam),(NMHDR*)(lParam)),0);
		case WM_LBUTTONDOWN:
			return (OnLButtonDown(hwnd,FALSE,(int)(short)LOWORD(lParam),(int)(short)HIWORD(lParam),(UINT)(wParam)),0);
		case WM_LBUTTONUP:
			return (OnLButtonUp(hwnd,(int)(short)LOWORD(lParam),(int)(short)HIWORD(lParam),(UINT)(wParam)),0);
		case WM_MOUSEMOVE:
			return (OnMouseMove(hwnd,(int)(short)LOWORD(lParam),(int)(short)HIWORD(lParam),(UINT)(wParam)),0);
		case WM_CREATE:
			return OnCreate((hwnd), (LPCREATESTRUCT) (lParam)), 0;
		case WM_DESTROY:
			return OnDestroy(hwnd), 0;

		default:
			return DefWindowProc(hwnd, wMsg, wParam, lParam);
	}
}

static BOOL OnCreate(HWND hWnd, LPCREATESTRUCT lpCreateStruct)
{
	InitCommonControls();
	g_hToolbar = MakeToolbar(hWnd, IDC_TOOLBAR);
	g_hStatus = MakeStatusbar(hWnd, IDC_STATUS);
	g_hMenu = MakeMenubar(hWnd);
	g_hWndTV = MakeTreeView(hWnd, IDC_TREE);
	g_hWndLV = MakeListView(hWnd, IDC_LIST);
	g_hWndHV = MakeListViewHex(hWnd, IDC_HVIEW);
	g_hWndHV2 = MakeListViewHex2(hWnd, IDC_HVIEW2);
	g_hWndLVC = g_hWndHV;	// Current ListView
	SetFocus(g_hWndTV);
	return 0;
}

static void OnDestroy(HWND hwnd)
{
	PostQuitMessage(0);
}

static void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch (id)
	{
		case IDM_OPEN:
			SetWindowText(g_hStatus, "");
			if (DoOpenFileName(hwnd))
				DoOpenFile(hwnd);
			return;

		case IDM_EXIT:
			PostMessage(hwnd, WM_CLOSE, 0, 0L);
			return;
	}

}

static void ReSizeWindows(HWND hWnd)
{
	RECT rc, rct, rcs;

	GetClientRect(hWnd, &rc);
	GetClientRect(g_hToolbar, &rct);
	GetClientRect(g_hStatus, &rcs);
	MoveWindow(g_hWndTV, 1, rct.bottom+2, uDragX-2, rc.bottom-rct.bottom-rcs.bottom-2, TRUE);
	MoveWindow(g_hWndLVC, uDragX+2, rct.bottom+2, rc.right-uDragX-2,  rc.bottom-rct.bottom-rcs.bottom-2, TRUE);
}

static void OnSize(HWND hWnd, UINT state, int cx, int cy)
{
	if (state == SIZE_MINIMIZED)
		return;
	SendMessage(g_hToolbar, WM_SIZE, state, cx);
	SendMessage(g_hStatus, WM_SIZE, state, cx);
	ReSizeWindows(hWnd);
}

static void OnNotify(HWND hwnd, int idCtrl, NMHDR* pNMHdr)
{
/*
	switch (pNMHdr->code)
	{
		case TTN_NEEDTEXT:
			//LPTOOLTIPTEXT lpttt;
			//lpttt = (LPTOOLTIPTEXT)pNMHdr;
			//lpttt->hinst = hInst;
			//lpttt->lpszText = szBuf;
			//LoadString(hInst, lpttt->hdr.idFrom, szBuf, sizeof(szBuf));
			return;
		case TVN_SELCHANGED:
			// NMTREEVIEW -> TVITEM -> TVITEM itemNew; -> HTREEITEM hItem;
			//itemNew = ((NMTREEVIEW*)pNMHdr)->itemNew;
			//TreeView_GetSelection(hWndTV);
			DoTreeViewSelChng();
			return;
	}*/
	if (idCtrl == IDC_TREE) {
		if (pNMHdr->code == TVN_SELCHANGED || pNMHdr->code == NM_DBLCLK)
			DoTreeViewSelChng();
		return;
	}
	if (idCtrl == IDC_LIST) {
		if (pNMHdr->code == NM_DBLCLK) {
			LVCellClick((LPARAM)pNMHdr);
		}
		return;
	}
	if (idCtrl == IDC_HVIEW) {
		ListViewNotify(hwnd, pNMHdr);
		return;
	}
	if (idCtrl == IDC_HVIEW2) {
		ListViewNotify2(hwnd, pNMHdr);
		return;
	}
}

static void OnLButtonDown(HWND hWnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	int dx;
	SetCapture(hWnd);

	dx = x - uDragX;
	if (dx > - 10 && dx < 10)
		bDragX = TRUE;
}

static void OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
	ReleaseCapture();
	bDragX = FALSE;
}

static void OnMouseMove(HWND hWnd, int x, int y, UINT keyFlags)
{
	RECT rc;

	if (bDragX)
	{
		GetClientRect(hWnd, &rc);
		if ((x > 2) && x < (rc.right - 2))
		{
			ReSizeWindows(hWnd);
			uDragX = x;
		}
	}
}

int HexToInt(char *pStr, int nLen);

static void LVCellClick(LPARAM lParam)
{
	HWND hList;
	LVHITTESTINFO lvhi;
	hList = ((LPNMHDR)lParam)->hwndFrom;
	lvhi.pt.x = ((LPNMITEMACTIVATE)lParam)->ptAction.x;
	lvhi.pt.y = ((LPNMITEMACTIVATE)lParam)->ptAction.y;
	ListView_SubItemHitTest(hList, &lvhi);
	//nRow = ((LPNMITEMACTIVATE)lParam)->iItem;
	int nRow = lvhi.iItem;
	//nCol = ((LPNMITEMACTIVATE)lParam)->iSubItem;
	int nCol = lvhi.iSubItem;
	if (nRow < 0 || nCol < 0) return;
	if (nCol == 3) {
		LVITEM lvi;
		char szTmp[50];
		lvi.mask =  LVIF_TEXT;
		lvi.iSubItem = nCol;
		lvi.pszText = szTmp;
		lvi.cchTextMax = sizeof(szTmp);
		int nLen = SendMessage(hList, LVM_GETITEMTEXT, nRow, (LPARAM)&lvi);
		//ListView_GetItemText(hList, nRow, nCol, szTmp, sizeof(szTmp));
		if (*(szTmp+nLen-1) == 'h') {
			//OutputDebugString(szTmp);
			DWORD nPos = HexToInt(szTmp, nLen);
			g_pHex = g_pData+nPos;
			g_nHexMax = g_nBlockSize;
			ListView_SetItemCount(g_hWndHV, (g_nBlockSize + 15) / 16);
			SendMessage(g_hWndHV, LVM_ENSUREVISIBLE, 0, 0);
			ShowWindow(g_hWndLV, SW_HIDE);
			ShowWindow(g_hWndHV2, SW_HIDE);
			ShowWindow(g_hWndHV, SW_SHOW);
			g_hWndLVC = g_hWndHV;
			SendMessage(g_hWndHV, LVM_SETCOLUMNWIDTH, 1, LVSCW_AUTOSIZE);
			SendMessage(g_hWndHV, LVM_SETCOLUMNWIDTH, 2, LVSCW_AUTOSIZE);
		}
	}
}

static int DoOpenFileName(HWND hWnd)
{
	static OPENFILENAME ofn = { 0 };
//	static char *szFilter = "All PE files (*.exe,*.dll,*.obj,*.o,*.lib,*.a,*.dbg,*.pdb)\0*.exe;*.dll;*.obj;*.o;*.lib;*.a;*.dbg;*.pdb\0All files (*.*)\0*.*\0\0";
	static char *szFilter = "pdb files (*.pdb)\0*.pdb\0All files (*.*)\0*.*\0\0";
//	char szFilter[100];     // for LoadString
	//static char szFileName[MAX_PATH], szFileTitle[MAX_PATH];

	if (!ofn.lStructSize)
	{
		ofn.lStructSize = sizeof(OPENFILENAME);
		ofn.hwndOwner = hWnd;
		//ofn.hInstance = hInst;
		ofn.lpstrFilter = szFilter;
		ofn.lpstrDefExt = szFilter;
		ofn.nFilterIndex = 1;
		ofn.lpstrFile = g_szFileName;
		ofn.nMaxFile = sizeof(g_szFileName);
		ofn.lpstrFileTitle = g_szFileTitle;
		//ofn.lpstrFileTitle = NULL;
		ofn.nMaxFileTitle = sizeof(g_szFileTitle);
		//ofn.nMaxFileTitle = 0;
		ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_EXPLORER | OFN_HIDEREADONLY;	// Open
#if (_WIN32_WINNT >= 0x0500)
		ofn.FlagsEx = OFN_EX_NOPLACESBAR;
#endif
		g_szFileName[0] = 0;
	}
	return GetOpenFileName(&ofn);
}

int Insert2TreePdb(BYTE *pData, DWORD nPos, DWORD nMax);

static int DoOpenFile(HWND hWnd)
{
	HANDLE hFile, hHeap;
	DWORD nRead, nMax;
	hHeap = GetProcessHeap();
	if (g_pFInfo) {
		HeapFree(hHeap, 0, g_pFInfo);	// free it
		g_pFInfo = NULL;	// mark it empty
	}

	if (g_pData)
	{	// if it exist
		HeapFree(hHeap, 0, g_pData);	// free it
		g_pData = NULL;	// mark it empty
	}
	if (g_pStrmBlockCnt1)
	{	// if it exist
		HeapFree(hHeap, 0, g_pStrmBlockCnt1);	// free it
		g_pStrmBlockCnt1 = NULL;	// mark it empty
	}
	if (g_pStrmBlockNum1)
	{	// if it exist
		HeapFree(hHeap, 0, g_pStrmBlockNum1);	// free it
		g_pStrmBlockNum1 = NULL;	// mark it empty
	}

	TreeViewClear(g_hWndTV);
	ListView_DeleteAllItems(g_hWndLV);
	ListView_DeleteAllItems(g_hWndHV);
	ListView_DeleteAllItems(g_hWndHV2);

	if ((hFile = CreateFile(g_szFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL)) != INVALID_HANDLE_VALUE)
	{
		nMax = GetFileSize(hFile, NULL);
		g_pData = HeapAlloc(hHeap, 0, nMax + 1);
		if (ReadFile(hFile, g_pData, nMax, &nRead, NULL))
		{
			CloseHandle(hFile);
			g_nFileSize = nRead;
			g_pFInfo = HeapAlloc(hHeap, HEAP_ZERO_MEMORY, MEMSIZE*sizeof(FINFO));
			g_pFInfo[0].Size = MEMSIZE;
			g_pFInfo[0].Ptr = (DWORD)g_pFInfo;	/// error pointer truncation

			g_pFInfo[1].Ptr = (UINT_PTR)g_pData;
			g_pFInfo[1].Size = nRead;
			g_pFInfo[1].Info = 0;
			g_hTVRoot = TreeViewInsert(NULL, TVI_ROOT, g_szFileTitle, 1);

			Insert2TreePdb(g_pData, 2, nMax);
			TreeView_Expand(g_hWndTV, g_hTVRoot, TVE_EXPAND);
		}
	}
	return 0;
}

static BOOL CheckMsgQueue(HWND hWnd)
{
	MSG msg;
	while (PeekMessage(&msg, hWnd, 0, 0, PM_REMOVE))
		if (msg.message == WM_QUIT)
			return TRUE;
		else
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	return FALSE;
}

static BOOL TreeViewClear(HWND hTree)
{
	int nC;
	int nCnt = TreeView_GetCount(hTree);
	for (nC = 0; nC < nCnt; nC++)
		TreeView_DeleteAllItems(hTree);
	return TRUE;
}

static HTREEITEM TreeViewInsert(HTREEITEM hParent, HTREEITEM hInsertAfter, const char *szText, LPARAM lParam)
{
	TVINSERTSTRUCT tvins;
	tvins.hParent = hParent;
	tvins.hInsertAfter = hInsertAfter;
	tvins.item.mask = TVIF_TEXT | TVIF_PARAM;
	tvins.item.pszText = (char *)szText;
	tvins.item.lParam = lParam;
	CheckMsgQueue(g_hFrame);
	return (HTREEITEM)SendMessage(g_hWndTV, TVM_INSERTITEM, 0, (LPARAM)&tvins);
}

static void ShowFileInfo(PFINFO pFInfo1);

static void DoTreeViewSelChng(void)
{
	//static 
	TV_ITEM tvi;
	HTREEITEM hTree;
	TNODE TNode;
	char szTmp[255], szPtr[50];

	// reset align change from library member header
	LVCOLUMN lvc;
	lvc.mask = LVCF_FMT;
	lvc.fmt = LVCFMT_RIGHT;
	SendMessage(g_hWndLV, LVM_SETCOLUMN, 1, (LPARAM)&lvc);
	g_hWndLVC = g_hWndLV;	// reset viewer

	//tvi.mask = TVIF_TEXT;
	//tvi.mask = TVIF_PARAM;
	tvi.mask = TVIF_TEXT | TVIF_PARAM;
	tvi.pszText = szTmp;
	tvi.cchTextMax = sizeof(szTmp);
	//tvi.hItem = TreeView_GetSelection(hWndTV);
	hTree = TreeView_GetSelection(g_hWndTV);
	tvi.hItem = hTree;
	TreeView_GetItem(g_hWndTV, &tvi);
	if (!tvi.lParam) {
		SetWindowText(g_hStatus, "");
		return;
	}
//	XTRACE("sc %p %u", g_pFInfo, g_pFInfo[0].Size);
	TNode = *(TNODE*)&tvi.lParam;
	if (tvi.lParam && (BYTE)TNode.Info != INFOLIBMH) {
		wsprintf(szPtr, " %08X", g_pFInfo[TNode.Pos].Ofs);
		lstrcat(szTmp, szPtr);
	}
	SetWindowText(g_hStatus, szTmp);
	wsprintf(szTmp, "%u %u", TNode.Pos, (BYTE)TNode.Info);
	SendMessage(g_hStatus, SB_SETTEXT, 1, (LPARAM)szTmp);
	if (hTree == g_hTVRoot) {
		g_pHex = g_pData;
		g_nHexMax = g_nFileSize;
		ListView_SetItemCount(g_hWndHV, (g_nFileSize + 15) / 16);
		SendMessage(g_hWndHV, LVM_ENSUREVISIBLE, 0, 0);
		ShowWindow(g_hWndLV, SW_HIDE);
		ShowWindow(g_hWndHV2, SW_HIDE);
		ShowWindow(g_hWndHV, SW_SHOW);
		g_hWndLVC = g_hWndHV;
/*		SendMessage(g_hWndHV, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
		SendMessage(g_hWndHV, LVM_SETCOLUMNWIDTH, 1, LVSCW_AUTOSIZE);
		SendMessage(g_hWndHV, LVM_SETCOLUMNWIDTH, 2, LVSCW_AUTOSIZE);
		SendMessage(g_hWndHV, LVM_SETCOLUMNWIDTH, 3, LVSCW_AUTOSIZE);*/
	} else {
		if (!TNode.Info || TNode.Info > 255 ) {	// not info -> just hexview
			//g_pHex = (PBYTE)((PFINFO)tvi.lParam)->Ptr;
			g_pHex = (PBYTE)g_pFInfo[TNode.Pos].Ptr;
			g_nHexMax = g_pFInfo[TNode.Pos].Size;
			ListView_SetItemCount(g_hWndHV, (g_pFInfo[TNode.Pos].Size + 15) / 16);
			SendMessage(g_hWndHV, LVM_ENSUREVISIBLE, 0, 0);
			ShowWindow(g_hWndLV, SW_HIDE);
			ShowWindow(g_hWndHV2, SW_HIDE);
			ShowWindow(g_hWndHV, SW_SHOW);
			g_hWndLVC = g_hWndHV;
		} else if (TNode.Info == -1 ) {	// just hexview 2
			g_pHex2 = (PBYTE)g_pFInfo[TNode.Pos].Ptr;
			g_pStreamBlocks2 = (DWORD*)g_pFInfo[TNode.Pos].Ptr2;
			g_nHexMax2 = g_pFInfo[TNode.Pos].Size;
			g_nBlkIdx = (DWORD)-1;	// mark not set
			//g_pStreamBlocks = (DWORD*)g_pFInfo[TNode.Pos].Ptr2;
			ListView_SetItemCount(g_hWndHV2, (g_pFInfo[TNode.Pos].Size + 15) / 16);
			SendMessage(g_hWndHV2, LVM_ENSUREVISIBLE, 0, 0);
			ShowWindow(g_hWndLV, SW_HIDE);
			ShowWindow(g_hWndHV, SW_HIDE);
			ShowWindow(g_hWndHV2, SW_SHOW);
			g_hWndLVC = g_hWndHV2;
		} else {
			ShowWindow(g_hWndHV, SW_HIDE);
			ShowWindow(g_hWndHV2, SW_HIDE);
			ShowWindow(g_hWndLV, SW_SHOW);
			g_hWndLVC = g_hWndLV;
			//LockWindowUpdate(hWndLV);
			//ShowFileInfo((PFINFO)tvi.lParam);
			ShowFileInfo(&g_pFInfo[TNode.Pos]);
			//LockWindowUpdate(NULL);
		}
		if (g_hWndLVC != g_hWndHV) {
			SendMessage(g_hWndLVC, LVM_SETCOLUMNWIDTH, 0, LVSCW_AUTOSIZE);
			SendMessage(g_hWndLVC, LVM_SETCOLUMNWIDTH, 1, LVSCW_AUTOSIZE);
			SendMessage(g_hWndLVC, LVM_SETCOLUMNWIDTH, 2, LVSCW_AUTOSIZE);
			SendMessage(g_hWndLVC, LVM_SETCOLUMNWIDTH, 3, LVSCW_AUTOSIZE);
		}
	}
	ReSizeWindows(g_hFrame);
}

void ListViewInsert(char *szPos, char *szData, char *szDesc, char *szValue)
{
	LV_ITEM lvi;
	lvi.mask = LVIF_TEXT;
	lvi.pszText = szPos;
	lvi.iItem = ListView_GetItemCount(g_hWndLV);
	lvi.iSubItem = 0;
	ListView_InsertItem(g_hWndLV, &lvi);
	if (szData) {
		lvi.iSubItem = 1;
		lvi.pszText = szData;
		ListView_SetItem(g_hWndLV, &lvi);
	}
	if (szDesc) {
		lvi.iSubItem = 2;
		lvi.pszText = szDesc;
		ListView_SetItem(g_hWndLV, &lvi);
	}
	if (szValue) {
		lvi.iSubItem = 3;
		lvi.pszText = szValue;
		ListView_SetItem(g_hWndLV, &lvi);
	}
}

char *IntToStr(int n, char *s)
{
	int i, sign, idx, nl;

	idx = 0;
	if ((sign = n) < 0) {	/* record sign */
		n = -n;	/* make n positive */
		idx++;
	}
	i = 0;
	nl = n;
	while ((nl /= 10) > 0)	/* count nums */
		idx++;
	s[idx+1] = '\0';
	do {	/* generate digits in reverse order */
		s[idx--] = n % 10 + '0';	/* get next digit */
	} while ((n /= 10) > 0);	/* delete it */
	if (sign < 0)
		s[0] = '-';
	return s;
}

char *IntToHex(int nNum, char *szBuf, int nLen)
{
	int i = nLen;
	szBuf[i] = 0;
	while (nNum != 0)
	{
		int j = nNum & 0xF;
		i--;
		if (j >= 10)
			szBuf[i] = (char)('A' + (j - 10));
		else
			szBuf[i] = (char)('0' + (j));
		if (!i) break;
		nNum = nNum >> 4;
	}
	while(i)
		szBuf[--i] = '0';
	return szBuf;
}

char *Str8ToHexBytes(char *szStr, char *szBuf, int nLen)
{
	int i = 0;
	unsigned long long nNum = *(unsigned long long*)szStr;
	int nLen3 = nLen * 3;
	while (nNum != 0)
	{
		int j = nNum & 0xF;
		if (j >= 10)
			szBuf[i+1] = (char)('A' + (j - 10));
		else
			szBuf[i+1] = (char)('0' + (j));
		nNum = nNum >> 4;
		j = nNum & 0xF;
		if (j >= 10)
			szBuf[i] = (char)('A' + (j - 10));
		else
			szBuf[i] = (char)('0' + (j));
		nNum = nNum >> 4;
		szBuf[i+2] = ' ';
		i+=3;
		if (i > nLen3) break;	// asked count only
	}
	for (;i < nLen*3; i+=3) {	// fill rest zeroes
		szBuf[i] = '0';
		szBuf[i+1] = '0';
		szBuf[i+2] = ' ';
	}
	szBuf[nLen*3-1] = 0;
	return szBuf;
}

int HexToInt(char *pStr, int nLen)
{
	int x = 0;
	char *pc = pStr+nLen-1;
	*pc = 0;	// h off
	while (*pc != ' ') pc--;
	pc++;
	do {
		if (*pc >= '0' && *pc <= '9') x += *pc - '0';
		else if (*pc >= 'A' && *pc <= 'F') x += *pc - '7';
		pc++;
		if (*pc) x *= 16;
	} while(*pc);
	return x;
}

static TCHAR *HexString(int nItem);
static TCHAR *ValString(int nItem);

static LRESULT ListViewNotify(HWND hWnd, NMHDR* pNMHdr)
{
	switch (pNMHdr->code)
	{
		case LVN_GETDISPINFO:
		{
			LV_DISPINFO *lpdi = (LV_DISPINFO *)pNMHdr;
			TCHAR szTmp[255];

			if (lpdi->item.iSubItem)
			{
				if (lpdi->item.mask & LVIF_TEXT)
				{
					if (lpdi->item.iSubItem == 1)
						lstrcpyn(lpdi->item.pszText, HexString(lpdi->item.iItem), lpdi->item.cchTextMax);
					else if (lpdi->item.iSubItem == 2)
						lstrcpyn(lpdi->item.pszText, ValString(lpdi->item.iItem), lpdi->item.cchTextMax);
				}
			}
			else
			{
				if (lpdi->item.mask & LVIF_TEXT)
				{
					//wsprintf(szTmp, TEXT("%08X"), lpdi->item.iItem * 16 + g_pHex - g_pData);
					if (g_fOpt & 1) IntToHex(lpdi->item.iItem * 16, szTmp, 8);
					else IntToHex(lpdi->item.iItem * 16 + g_pHex - g_pData, szTmp, 8);
					lstrcpyn(lpdi->item.pszText, szTmp, lpdi->item.cchTextMax);
				}

				if (lpdi->item.mask & LVIF_IMAGE)
				{
					lpdi->item.iImage = 0;
				}
			}
		}
			return 0;
	}
	return 0;
}

static TCHAR *HexString(int nItem)
{
	BYTE *pPtr;
	TCHAR *szHex = "0123456789ABCDEF";
	static TCHAR szHexStr[50];
	int nC;

	pPtr = g_pHex + nItem * 16;
	szHexStr[0] = 0;
	for (nC = 0; nC < 16; nC++)
	{
		int nOfs;
		nOfs = nC * 3;
		if (nC > 7)
			szHexStr[nOfs++] = ' ';
		if ((pPtr + nC) >= (g_pHex + g_nHexMax)) {
			szHexStr[nOfs] = 0;
			break;
		}
		szHexStr[nOfs] = szHex[(*(pPtr + nC) & 0xF0) >> 4];
		szHexStr[nOfs + 1] = szHex[(*(pPtr + nC) & 0x0F)];
		szHexStr[nOfs + 2] = ' ';
	}
	szHexStr[48] = 0;
	return szHexStr;
}

static TCHAR *ValString(int nItem)
{
	BYTE *pPtr;
	static TCHAR szValStr[25];
	int nC;
	pPtr = g_pHex + nItem * 16;
	szValStr[0] = 0;
	for (nC = 0; nC < 16; nC++)
	{
		if ((pPtr + nC) >= (g_pHex + g_nHexMax)) {
			szValStr[nC] = 0;
			break;
		}
		if (*(pPtr + nC) > 31)
			szValStr[nC] = *(pPtr + nC);
		else
			szValStr[nC] = '.';
	}
	szValStr[16] = 0;
	return szValStr;
}

static TCHAR *HexString2(int nItem, BYTE *pHex, int nLeft)
{
	BYTE *pPtr;
	TCHAR *szHex = "0123456789ABCDEF";
	static TCHAR szHexStr[50];
	int nC;

	pPtr = pHex + nItem;	// * 16;
	szHexStr[0] = 0;
	for (nC = 0; nC < 16; nC++)
	{
		int nOfs;
		nOfs = nC * 3;
		if (nC > 7)
			szHexStr[nOfs++] = ' ';
//		if ((pPtr + nC) >= (pHex + g_nHexMax2)) {
		if (nC > nLeft) {
			szHexStr[nOfs] = 0;
			break;
		}
		szHexStr[nOfs] = szHex[(*(pPtr + nC) & 0xF0) >> 4];
		szHexStr[nOfs + 1] = szHex[(*(pPtr + nC) & 0x0F)];
		szHexStr[nOfs + 2] = ' ';
	}
	szHexStr[48] = 0;
	return szHexStr;
}

static TCHAR *ValString2(int nItem, BYTE *pHex, int nLeft)
{
	BYTE *pPtr;
	static TCHAR szValStr[25];
	int nC;
	pPtr = pHex + nItem;	// * 16;
	szValStr[0] = 0;
	for (nC = 0; nC < 16; nC++)
	{
//		if ((pPtr + nC) >= (pHex + g_nHexMax2)) {
		if (nC > nLeft) {
			szValStr[nC] = 0;
			break;
		}
		if (*(pPtr + nC) > 31)
			szValStr[nC] = *(pPtr + nC);
		else
			szValStr[nC] = '.';
	}
	szValStr[16] = 0;
	return szValStr;
}

char *CTime(DWORD dwTimeDateStamp)
{
	static char szTimeDateStamp[25];
	if (dwTimeDateStamp) {
		LONGLONG ft = (dwTimeDateStamp + 11644473600) * 10000000;
		SYSTEMTIME st;
		FileTimeToSystemTime((FILETIME*)&ft,&st);
		wsprintf(szTimeDateStamp,"%04u-%02u-%02u %02u:%02u:%02u UTC",
			st.wYear,st.wMonth,st.wDay,st.wHour,st.wMinute,st.wSecond);
	} else *szTimeDateStamp = 0;
	return szTimeDateStamp;
}

static LRESULT ListViewNotify2(HWND hWnd, NMHDR* pNMHdr)
{
	switch (pNMHdr->code)
	{
		case LVN_GETDISPINFO:
		{
			LV_DISPINFO *lpdi = (LV_DISPINFO *)pNMHdr;
			TCHAR szTmp[255];
			if ((DWORD)(lpdi->item.iItem*16 / g_nBlockSize) != g_nBlkIdx
				 || g_nBlkIdx == (DWORD)-1) {	// point next block ?
				g_nBlkIdx = lpdi->item.iItem*16 / g_nBlockSize;
				g_pHex2 = (PBYTE)g_pData+(g_pStreamBlocks2[g_nBlkIdx]*g_nBlockSize);
			}
			if (lpdi->item.iSubItem)
			{
				if (lpdi->item.mask & LVIF_TEXT)
				{
					int nLeft = g_nHexMax2 - lpdi->item.iItem*16;
					//if (nLeft < 16) OutputDebugString("nLeft < 16");
					DWORD nOfs = (lpdi->item.iItem * 16) % g_nBlockSize;
					if (lpdi->item.iSubItem == 1)
						lstrcpyn(lpdi->item.pszText, HexString2(nOfs, g_pHex2, nLeft), lpdi->item.cchTextMax);
					else if (lpdi->item.iSubItem == 2)
						lstrcpyn(lpdi->item.pszText, ValString2(nOfs, g_pHex2, nLeft), lpdi->item.cchTextMax);
				}
			}
			else
			{
				if (lpdi->item.mask & LVIF_TEXT)
				{
					DWORD nOfs = (lpdi->item.iItem % 16) * 16;
					//if (g_fOpt & 1) IntToHex(lpdi->item.iItem * 16, szTmp, 8);
					//else IntToHex(lpdi->item.iItem * 16 + g_pHex2 - g_pData, szTmp, 8);
					if (g_fOpt & 1) IntToHex(nOfs, szTmp, 8);
					else IntToHex(nOfs + g_pHex2 - g_pData, szTmp, 8);
					lstrcpyn(lpdi->item.pszText, szTmp, lpdi->item.cchTextMax);
				}

				if (lpdi->item.mask & LVIF_IMAGE)
				{
					lpdi->item.iImage = 0;
				}
			}
		}
			return 0;
	}
	return 0;
}

int CheckFInfo(DWORD nPos)
{
//#define MEMSIZE 10000
//	PFINFO pFInfo = g_pFInfo;
	if (nPos >= g_pFInfo->Size) {	/// no success with LocalReAlloc
		void *p = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (void*)g_pFInfo->Ptr, (g_pFInfo->Size+MEMSIZE)*sizeof(FINFO));
		if (p) {
			g_pFInfo = (PFINFO)p;
			//g_pFInfo->Ptr = (DWORD_PTR)p;
			//g_pFInfo->Size += MEMSIZE;
			g_pFInfo[0].Ptr = (DWORD_PTR)p;
			g_pFInfo[0].Size += MEMSIZE;
		} else
			return 0;
	}
	return g_pFInfo->Size;
}

void ShowInfo_PdbHeader(PFINFO pFInfo1);	/// msf/pdb header
void ShowInfo_PdbStreamDir(PFINFO pFInfo1);	/// msf/pdb stream dir
void ShowInfo_PdbStreamHdr(PFINFO pFInfo1);	/// msf/pdb stream header
void ShowInfo_PdbTpiHdr(PFINFO pFInfo1);	/// 122 pdb tpi header
void ShowInfo_PdbDbiHdr(PFINFO pFInfo1);	/// 123 pdb dbi header
void ShowInfo_HexView2(PFINFO pFInfo1);	/// msf/pdb block hexview

static void ShowFileInfo(PFINFO pFInfo1)
{
	BYTE Info = pFInfo1->Info;
	ListView_DeleteAllItems(g_hWndLV);
	if (Info == 119) {	/// pdb header
		ShowInfo_PdbHeader(pFInfo1);
		return;
	}
	if (Info == 120) {	/// pdb stream dir
		ShowInfo_PdbStreamDir(pFInfo1);
		return;
	}
	if (Info == 121) {	/// pdb stream header
		ShowInfo_PdbStreamHdr(pFInfo1);
		return;
	}
	if (Info == 122) {	/// pdb tpi header
		ShowInfo_PdbTpiHdr(pFInfo1);
		return;
	}
	if (Info == 123) {	/// pdb dbi header
		ShowInfo_PdbDbiHdr(pFInfo1);
		return;
	}
}

// PDB part
#include <stdint.h>
const char Magic[] = "Microsoft C/C++ MSF 7.00\r\n\x1a\x44\x53\0\0";

typedef struct SuperBlock {
  char FileMagic[32];	//sizeof(Magic)];
  uint32_t BlockSize;
  uint32_t FreeBlockMapBlock;
  uint32_t NumBlocks;
  uint32_t NumDirectoryBytes;
  uint32_t Unknown;
  uint32_t BlockMapAddr;
}SUPERBLOCK,*PSUPERBLOCK;

typedef struct StreamDirectory {
  uint32_t NumStreams;
  uint32_t StreamSizes;	//[NumStreams];
  uint32_t StreamBlocks[];	//[NumStreams][];
}STREAMDIRECTORY,*PSTREAMDIRECTORY;

typedef struct PdbStreamHeader {
  uint32_t Version;	// VC70 20000404 1312E94h
  uint32_t Signature;
  uint32_t Age;
  GUID UniqueId;
}PDBSTREAMHEADER,*PPDBSTREAMHEADER;

typedef struct DbiStreamHeader {
  int32_t VersionSignature;
  uint32_t VersionHeader;
  uint32_t Age;
  uint16_t GlobalStreamIndex;
  uint16_t BuildNumber;
  uint16_t PublicStreamIndex;
  uint16_t PdbDllVersion;
  uint16_t SymRecordStream;
  uint16_t PdbDllRbld;
  int32_t ModInfoSize;
  int32_t SectionContributionSize;
  int32_t SectionMapSize;
  int32_t SourceInfoSize;
  int32_t TypeServerSize;
  uint32_t MFCTypeServerIndex;
  int32_t OptionalDbgHeaderSize;
  int32_t ECSubstreamSize;
  uint16_t Flags;
  uint16_t Machine;
  uint32_t Padding;
}DBISTREAMHEADER, *PDBISTREAMHEADER;

/*
Stream No.	Contents	Short Description
1	Pdb (header)	Version information, and information to connect this PDB to the EXE
2	Tpi (Type manager)	All the types used in the executable.
3	Dbi (Debug information)	Holds section contributions, and list of Mods
4	NameMap	Holds a hashed string table
4-(n+4)	n Mods (Module information)	Each Mod stream holds symbols and line numbers for one compiland
n+4	Global symbol hash	An index that allows searching in global symbols by name
n+5	Public symbol hash	An index that allows searching in public symbols by addresses
n+6	Symbol records	Actual symbol records of global and public symbols
n+7	Type hash	Hash used by the TPI stream.
*/
char *aStrmName[] = {", Old MSF directory", ", PDB header",
 ", TPI (Type info)", ", DBI (Debug Info)", ", IPI (Name Map)"};

int CheckBlocks(DWORD *pStreamBlocks, DWORD nBlocks)
{
	DWORD nBlock = pStreamBlocks[0];
	for (DWORD nC = 1; nC < nBlocks; nC++) {
		if (pStreamBlocks[nC] != nBlock + 1) {
			return 0;
		}
		nBlock = pStreamBlocks[nC];
	}
	return pStreamBlocks[0];
}

int Insert2TreePdb(BYTE *pData, DWORD nPos, DWORD nMax)
{
	SUPERBLOCK *pMsfHdr = (SUPERBLOCK*)pData;
	if (memcmp(pMsfHdr->FileMagic, Magic, 30)) return 0;

	char szTmp[255];
	DWORD nOfs = (DWORD)(pData-g_pData);
	TNODE TNode;
	*(LPARAM*)&TNode = 0;	// empty
	HTREEITEM hRoot = g_hTVRoot;
	DWORD nBlockSize = pMsfHdr->BlockSize;
	g_nBlockSize = nBlockSize;	// global for viewers
	g_pDbiHdr = 0;

	g_pFInfo[nPos].Ofs = nOfs;
	g_pFInfo[nPos].Ptr = (DWORD_PTR)pData;
	g_pFInfo[nPos].Size = sizeof(SUPERBLOCK);
	g_pFInfo[nPos].Info = 119;
	TNode.Pos = nPos++;
	TNode.Info = 119;
	TreeViewInsert(hRoot, TVI_LAST, "SuperBlock", (LPARAM)*(LPARAM*)&TNode);

	g_pFInfo[nPos].Ofs = nOfs;
	g_pFInfo[nPos].Ptr = (DWORD_PTR)(pData+pMsfHdr->BlockSize);
	g_pFInfo[nPos].Size = pMsfHdr->BlockSize;
	g_pFInfo[nPos].Info = 0;
	TreeViewInsert(hRoot, TVI_LAST, "FreeBlockMap 1", nPos++);	// hexview

	g_pFInfo[nPos].Ofs = nOfs;
	g_pFInfo[nPos].Ptr = (DWORD_PTR)(pData+pMsfHdr->BlockSize*2);
	g_pFInfo[nPos].Size = pMsfHdr->BlockSize;
	g_pFInfo[nPos].Info = 0;
	TreeViewInsert(hRoot, TVI_LAST, "FreeBlockMap 2", nPos++);	// hexview

	g_pFInfo[nPos].Ofs = nOfs;
	g_pFInfo[nPos].Ptr = (DWORD_PTR)(pData+pMsfHdr->BlockMapAddr*pMsfHdr->BlockSize);
	g_pFInfo[nPos].Size = (pMsfHdr->NumDirectoryBytes/pMsfHdr->BlockSize+1)*4;
	g_pFInfo[nPos].Info = 0;
	TreeViewInsert(hRoot, TVI_LAST, "Stream Directory Blocks", nPos++);	// hexview

	g_pFInfo[nPos].Ofs = *(pData+pMsfHdr->BlockMapAddr*pMsfHdr->BlockSize)*pMsfHdr->BlockSize;
	g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+*(DWORD*)(pData+pMsfHdr->BlockMapAddr*pMsfHdr->BlockSize)*pMsfHdr->BlockSize;
	g_pFInfo[nPos].Size = pMsfHdr->NumDirectoryBytes;
	g_pFInfo[nPos].Info = 120;
	TNode.Pos = nPos++;
	TNode.Info = 120;
	TreeViewInsert(hRoot, TVI_LAST, "Stream Directory", (LPARAM)*(LPARAM*)&TNode);
	//HTREEITEM hStrmDir =TreeViewInsert(hRoot, TVI_LAST, "Stream Directory", nPos++);	// hexview

	// init locals for streams
	DWORD *pIndexes = (DWORD*)(pData+pMsfHdr->BlockMapAddr*pMsfHdr->BlockSize);
	DWORD *pIndex = pIndexes;	// the first Index Block
	PSTREAMDIRECTORY pStreamDir = (PSTREAMDIRECTORY)(pData+*pIndex*pMsfHdr->BlockSize);
	DWORD nStreams = pStreamDir->NumStreams;
	//DWORD nStreamBlocks = pMsfHdr->NumDirectoryBytes/4-1-pStreamDir->NumStreams;
	g_pStrmBlockCnt1 = HeapAlloc(GetProcessHeap(), 0, nStreams*sizeof(DWORD));
	g_pStrmBlockNum1 = HeapAlloc(GetProcessHeap(), 0, nStreams*sizeof(DWORD));
	DWORD *pStreamSizes = (DWORD*)&pStreamDir->StreamSizes;
	DWORD *pStreamBlocks = (DWORD*)&pStreamDir->StreamSizes;
	pStreamBlocks += nStreams;
	g_pStreamSizes = pStreamSizes;
	g_pStreamBlocks = pStreamBlocks;	// for global access

	int nBlkCnt = 0;
	int nStream = 0;
	HTREEITEM hChild;
	for (DWORD nC=0; nC<nStreams; nC++) {
		int nBlocks = 0;
		if (pStreamSizes[nC]>0 && pStreamSizes[nC]<0xFFFFFFFF) {	// -1 free block
			nBlocks = (pStreamSizes[nC]-1)/nBlockSize+1;
		} else nBlocks = 0;
		int nBlk1 = nBlkCnt;	// save 1. block index
		int nLen;
		if (nBlocks)
			nLen = wsprintf(szTmp, "stream #%u %Xh", nStream, pStreamBlocks[nBlk1]*nBlockSize);
		else
			nLen = wsprintf(szTmp, "stream #%u", nStream);
		if (nStream <= 4) {
			strcpy(szTmp+nLen, aStrmName[nStream]);
		}
		int nChkSeq = 0;
		if (pStreamSizes[nC]) //{
			nChkSeq = CheckBlocks(&pStreamBlocks[nBlk1], nBlocks);	// are blocks sequential ?
		g_pFInfo[nPos].Ofs = pStreamBlocks[nBlk1]*nBlockSize;
		g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlk1]*nBlockSize;
		g_pFInfo[nPos].Ptr2 = (DWORD_PTR)&g_pStreamBlocks[nBlk1];
		g_pFInfo[nPos].Size = pStreamSizes[nC];	//nBlocks*nBlockSize;
		//if (pStreamSizes[nC] > nBlockSize && !CheckBlocks(&pStreamBlocks[nBlk1], nBlocks)) {	// alternate hex viewer 2
		if (pStreamSizes[nC] > nBlockSize && !nChkSeq) {	// use alternate hex viewer 2
			g_pFInfo[nPos].Info = 255;
			TNode.Pos = nPos++;
			TNode.Info = 255;
			hChild = TreeViewInsert(hRoot, TVI_LAST, szTmp, (LPARAM)*(LPARAM*)&TNode);	// hexview2
		} else {
			g_pFInfo[nPos].Info = 0;
			hChild = TreeViewInsert(hRoot, TVI_LAST, szTmp, nPos++);	// hexview
		}
		if (!nStream) {
			g_pFInfo[nPos].Ofs = pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr2 = (DWORD_PTR)&g_pStreamBlocks[nBlkCnt];
			g_pFInfo[nPos].Size = pStreamSizes[nC];
			g_pFInfo[nPos].Info = 120;
			//TreeViewInsert(hChild, TVI_LAST, "Old Root Directory", nPos++);	// hexview
			TNode.Pos = nPos++;
			TNode.Info = 120;
			TreeViewInsert(hChild, TVI_LAST, "Old MSF Directory", (LPARAM)*(LPARAM*)&TNode);
		} else if (nStream == 1 && pStreamSizes[nC]) {	// PDB Header
			g_pFInfo[nPos].Ofs = pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr2 = (DWORD_PTR)&g_pStreamBlocks[nBlkCnt];
			g_pFInfo[nPos].Size = pStreamSizes[nC];
			g_pFInfo[nPos].Info = 121;
			TNode.Pos = nPos++;
			TNode.Info = 121;
			TreeViewInsert(hChild, TVI_LAST, "Stream Header", (LPARAM)*(LPARAM*)&TNode);
		} else if (nStream == 2 && pStreamSizes[nC]) {	// TPI Header
			g_pFInfo[nPos].Ofs = pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr2 = (DWORD_PTR)&g_pStreamBlocks[nBlkCnt];
			g_pFInfo[nPos].Size = pStreamSizes[nC];
			g_pFInfo[nPos].Info = 122;
			TNode.Pos = nPos++;
			TNode.Info = 122;
			TreeViewInsert(hChild, TVI_LAST, "TPI Header", (LPARAM)*(LPARAM*)&TNode);
		} else if (nStream == 3 && pStreamSizes[nC]) {	// DBI Header
			g_pFInfo[nPos].Ofs = pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr2 = (DWORD_PTR)&g_pStreamBlocks[nBlkCnt];
			g_pFInfo[nPos].Size = pStreamSizes[nC];
			g_pFInfo[nPos].Info = 123;
			TNode.Pos = nPos++;
			TNode.Info = 123;
			TreeViewInsert(hChild, TVI_LAST, "DBI Header", (LPARAM)*(LPARAM*)&TNode);
			g_pDbiHdr = (PDBISTREAMHEADER)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
		} else if (nStream == 4 && pStreamSizes[nC]) {	// DBI Header
			g_pFInfo[nPos].Ofs = pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr2 = (DWORD_PTR)&g_pStreamBlocks[nBlkCnt];
			g_pFInfo[nPos].Size = pStreamSizes[nC];
			g_pFInfo[nPos].Info = 122;
			TNode.Pos = nPos++;
			TNode.Info = 122;
			TreeViewInsert(hChild, TVI_LAST, "IPI Header", (LPARAM)*(LPARAM*)&TNode);
			g_pDbiHdr = (PDBISTREAMHEADER)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
		}
		for (int i=0; i<nBlocks; i++) {
			wsprintf(szTmp, "block %u %Xh", pStreamBlocks[nBlkCnt], pStreamBlocks[nBlkCnt]*nBlockSize);
			g_pFInfo[nPos].Ofs = pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Ptr = (DWORD_PTR)pData+pStreamBlocks[nBlkCnt]*nBlockSize;
			g_pFInfo[nPos].Size = nBlockSize;
			g_pFInfo[nPos].Info = 0;
			TreeViewInsert(hChild, TVI_LAST, szTmp, nPos++);	// hexview
			//g_pStreamBlocks[nBlkCnt] = pStreamBlocks[nBlkCnt];	// save global block index
			nBlkCnt++;
		}
		g_pStrmBlockCnt1[nC] = nBlocks;	// save stream block count
		if (nBlkCnt) {
			g_pStrmBlockNum1[nC] = pStreamBlocks[nBlk1];	//nC];	// save stream first block number
		} else g_pStrmBlockNum1[nC] = 0;	// no blocks for it
		nStream++;
	}
	return nPos;
}

void ShowInfo_PdbHeader(PFINFO pFInfo1)
{
	char szPos[10], szData[25], szValue[100];
	PBYTE pPtr;
	if (g_fOpt & 1) pPtr = 0;
	else pPtr = (PBYTE)(pFInfo1->Ptr-(DWORD_PTR)g_pData);
	SUPERBLOCK *pMsfHdr = (SUPERBLOCK*)pFInfo1->Ptr;
	IntToHex((DWORD_PTR)pPtr, szPos, 8);
	Str8ToHexBytes((char*)pMsfHdr->FileMagic, szData, 8);
	ListViewInsert(szPos, szData, "FileMagic", pMsfHdr->FileMagic);
	IntToHex((int)pPtr+offsetof(SUPERBLOCK, BlockSize), szPos, 8);
	IntToHex(pMsfHdr->BlockSize, szData, 8);
	IntToStr(pMsfHdr->BlockSize, szValue);
	ListViewInsert(szPos, szData, "BlockSize", szValue);
	IntToHex((int)pPtr+offsetof(SUPERBLOCK, FreeBlockMapBlock), szPos, 8);
	IntToHex(pMsfHdr->FreeBlockMapBlock, szData, 8);
	//IntToStr(pMsfHdr->FreeBlockMapBlock, szValue);
	wsprintf(szValue, "%u -> %Xh", pMsfHdr->FreeBlockMapBlock, pMsfHdr->FreeBlockMapBlock*pMsfHdr->BlockSize);
	ListViewInsert(szPos, szData, "FreeBlockMapBlock", szValue);
	IntToHex((int)pPtr+offsetof(SUPERBLOCK, NumBlocks), szPos, 8);
	IntToHex(pMsfHdr->NumBlocks, szData, 8);
	IntToStr(pMsfHdr->NumBlocks, szValue);
	ListViewInsert(szPos, szData, "NumBlocks", szValue);
	IntToHex((int)pPtr+offsetof(SUPERBLOCK, NumDirectoryBytes), szPos, 8);
	IntToHex(pMsfHdr->NumDirectoryBytes, szData, 8);
	IntToStr(pMsfHdr->NumDirectoryBytes, szValue);
	ListViewInsert(szPos, szData, "NumDirectoryBytes", szValue);
	IntToHex((int)pPtr+offsetof(SUPERBLOCK, Unknown), szPos, 8);
	IntToHex(pMsfHdr->Unknown, szData, 8);
	IntToStr(pMsfHdr->Unknown, szValue);
	ListViewInsert(szPos, szData, "Unknown", szValue);
	IntToHex((int)pPtr+offsetof(SUPERBLOCK, BlockMapAddr), szPos, 8);
	IntToHex(pMsfHdr->BlockMapAddr, szData, 8);
	//IntToStr(pMsfHdr->BlockMapAddr, szValue);
	wsprintf(szValue, "%u -> %Xh", pMsfHdr->BlockMapAddr, pMsfHdr->BlockMapAddr*pMsfHdr->BlockSize);
	ListViewInsert(szPos, szData, "BlockMapAddr", szValue);
}

void ShowInfo_PdbStreamDir(PFINFO pFInfo1)
{
	char szPos[10], szData[25], szValue[100], szTmp[20];
	PBYTE pPtr;
	if (g_fOpt & 1) pPtr = 0;
	else pPtr = (PBYTE)(pFInfo1->Ptr-(DWORD_PTR)g_pData);
	DWORD nSizes = *(DWORD*)pFInfo1->Ptr;
	DWORD *pSize = (DWORD*)pFInfo1->Ptr;
	IntToHex((DWORD_PTR)pPtr, szPos, 8);
	IntToHex(nSizes, szData, 8);
	IntToStr(nSizes, szValue);
	ListViewInsert(szPos, szData, "Stream sizes", szValue);
	int nBlockSize = ((SUPERBLOCK*)g_pData)->BlockSize;
	int nBlkCnt = 0;
	int nStream = 0;
	for (DWORD nC = 0; nC < nSizes; nC++) {
		pSize++;
		pPtr += 4;
		if (*pSize && *pSize < 0xFFFFFFFF) {
			nBlkCnt += *pSize/nBlockSize+1;
			nStream++;
		}
		IntToHex((DWORD_PTR)pPtr, szPos, 8);
		IntToHex(*pSize, szData, 8);
		IntToStr(*pSize, szValue);
		IntToStr(nC, szTmp);
		ListViewInsert(szPos, szData, szTmp, szValue);
		//if (nC > 255) return;	// only 1 page
	}
	IntToHex((DWORD_PTR)(pPtr+4), szPos, 8);
	IntToStr(nBlkCnt, szValue);
	ListViewInsert(szPos, "", "Stream blocks", szValue);
	for (int nC = 0; nC < nBlkCnt; nC++) {
		pSize++;
		pPtr += 4;
		IntToHex((DWORD_PTR)pPtr, szPos, 8);
		IntToHex(*pSize, szData, 8);
		//IntToStr(*pSize, szValue);
		wsprintf(szValue, "%u -> %Xh", *pSize, *pSize*nBlockSize);
		IntToStr(nC, szTmp);
		ListViewInsert(szPos, szData, szTmp, szValue);
	}
}

void ShowInfo_PdbStreamHdr(PFINFO pFInfo1)	/// 121
{
	char szPos[10], szData[25], szValue[100];	//, szTmp[20];
	PBYTE pPtr;
	if (g_fOpt & 1) pPtr = 0;
	else pPtr = (PBYTE)(pFInfo1->Ptr-(DWORD_PTR)g_pData);
	PPDBSTREAMHEADER pStreamHeader = (PPDBSTREAMHEADER)pFInfo1->Ptr;
	IntToHex((DWORD_PTR)pPtr, szPos, 8);
	IntToHex(pStreamHeader->Version, szData, 8);
	IntToStr(pStreamHeader->Version, szValue);
	ListViewInsert(szPos, szData, "Version", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(PDBSTREAMHEADER, Signature), szPos, 8);
	IntToHex(pStreamHeader->Signature, szData, 8);
	IntToStr(pStreamHeader->Signature, szValue);
	ListViewInsert(szPos, szData, "Signature", CTime(pStreamHeader->Signature));
	IntToHex((DWORD_PTR)pPtr+offsetof(PDBSTREAMHEADER, Age), szPos, 8);
	IntToHex(pStreamHeader->Age, szData, 8);
	IntToStr(pStreamHeader->Age, szValue);
	ListViewInsert(szPos, szData, "Age", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(PDBSTREAMHEADER, UniqueId), szPos, 8);
	Str8ToHexBytes((char*)&pStreamHeader->UniqueId, szData, 8);
	//IntToStr(pStreamHeader->UniqueId, szValue);
	ListViewInsert(szPos, szData, "UniqueId", "");
	IntToHex((DWORD_PTR)(pPtr+offsetof(PDBSTREAMHEADER, UniqueId)+8), szPos, 8);
	Str8ToHexBytes(((char*)&pStreamHeader->UniqueId)+8, szData, 8);
	ListViewInsert(szPos, szData, "", "");
}

typedef struct TpiStreamHeader {
	uint32_t Version;
	uint32_t Size;
	uint32_t tiMin;
	uint32_t tiMac;
	uint32_t SizeGPRec;
}TPISTREAMHEADER, *PTPISTREAMHEADER;

void ShowInfo_PdbTpiHdr(PFINFO pFInfo1)	/// 122
{
	char szPos[10], szData[25], szValue[100];	//, szTmp[20];
	PBYTE pPtr;
	if (g_fOpt & 1) pPtr = 0;
	else pPtr = (PBYTE)(pFInfo1->Ptr-(DWORD_PTR)g_pData);
	PTPISTREAMHEADER pTpiHdr = (PTPISTREAMHEADER)pFInfo1->Ptr;
	IntToHex((DWORD_PTR)pPtr, szPos, 8);
	//IntToHex(*(DWORD*)pFInfo1->Ptr, szData, 8);
	//IntToStr(*(DWORD*)pFInfo1->Ptr, szValue);
	IntToHex(pTpiHdr->Version, szData, 8);
	IntToStr(pTpiHdr->Version, szValue);
	ListViewInsert(szPos, szData, "Version", szValue);
	IntToHex((DWORD_PTR)(pPtr+4), szPos, 8);
	IntToHex(pTpiHdr->Size, szData, 8);
	IntToStr(pTpiHdr->Size, szValue);
	ListViewInsert(szPos, szData, "Size", szValue);
	IntToHex((DWORD_PTR)(pPtr+8), szPos, 8);
	IntToHex(pTpiHdr->tiMin, szData, 8);
	IntToStr(pTpiHdr->tiMin, szValue);
	ListViewInsert(szPos, szData, "tiMin", szValue);
	IntToHex((DWORD_PTR)(pPtr+12), szPos, 8);
	IntToHex(pTpiHdr->tiMac, szData, 8);
	IntToStr(pTpiHdr->tiMac, szValue);
	ListViewInsert(szPos, szData, "tiMac", szValue);
	IntToHex((DWORD_PTR)(pPtr+16), szPos, 8);
	IntToHex(pTpiHdr->SizeGPRec, szData, 8);
	IntToStr(pTpiHdr->SizeGPRec, szValue);
	ListViewInsert(szPos, szData, "SizeGPRec", szValue);
}

void ShowInfo_PdbDbiHdr(PFINFO pFInfo1)	/// 123
{
	char szPos[10], szData[25], szValue[100];	//, szTmp[20];
	PBYTE pPtr;
	if (g_fOpt & 1) pPtr = 0;
	else pPtr = (PBYTE)(pFInfo1->Ptr-(DWORD_PTR)g_pData);
	PDBISTREAMHEADER pDbiHdr = (PDBISTREAMHEADER)pFInfo1->Ptr;
	IntToHex((DWORD_PTR)pPtr, szPos, 8);
	IntToHex(pDbiHdr->VersionSignature, szData, 8);
	IntToStr(pDbiHdr->VersionSignature, szValue);
	ListViewInsert(szPos, szData, "VersionSignature", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, VersionHeader), szPos, 8);
	IntToHex(pDbiHdr->VersionHeader, szData, 8);
	IntToStr(pDbiHdr->VersionHeader, szValue);
	ListViewInsert(szPos, szData, "VersionHeader", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, Age), szPos, 8);
	IntToHex(pDbiHdr->Age, szData, 8);
	IntToStr(pDbiHdr->Age, szValue);
	ListViewInsert(szPos, szData, "Age", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, GlobalStreamIndex), szPos, 8);
	IntToHex(pDbiHdr->GlobalStreamIndex, szData, 4);
	//IntToStr(pDbiHdr->GlobalStreamIndex, szValue);
	wsprintf(szValue, "%u -> %u ->%Xh", pDbiHdr->GlobalStreamIndex,
		g_pStrmBlockNum1[pDbiHdr->GlobalStreamIndex],
		g_pStrmBlockNum1[pDbiHdr->GlobalStreamIndex]*g_nBlockSize);
	ListViewInsert(szPos, szData, "GlobalStreamIndex", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, BuildNumber), szPos, 8);
	IntToHex(pDbiHdr->BuildNumber, szData, 4);
	IntToStr(pDbiHdr->BuildNumber, szValue);
	ListViewInsert(szPos, szData, "BuildNumber", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, PublicStreamIndex), szPos, 8);
	IntToHex(pDbiHdr->PublicStreamIndex, szData, 4);
	//IntToStr(pDbiHdr->PublicStreamIndex, szValue);
	wsprintf(szValue, "%u -> %u ->%Xh", pDbiHdr->PublicStreamIndex,
		g_pStrmBlockNum1[pDbiHdr->PublicStreamIndex],
		g_pStrmBlockNum1[pDbiHdr->PublicStreamIndex]*g_nBlockSize);
	ListViewInsert(szPos, szData, "PublicStreamIndex", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, PdbDllVersion), szPos, 8);
	IntToHex(pDbiHdr->PdbDllVersion, szData, 4);
	IntToStr(pDbiHdr->PdbDllVersion, szValue);
	ListViewInsert(szPos, szData, "PdbDllVersion", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, SymRecordStream), szPos, 8);
	IntToHex(pDbiHdr->SymRecordStream, szData, 4);
	//IntToStr(pDbiHdr->SymRecordStream, szValue);
	wsprintf(szValue, "%u -> %u ->%Xh", pDbiHdr->SymRecordStream,
		g_pStrmBlockNum1[pDbiHdr->SymRecordStream],
		g_pStrmBlockNum1[pDbiHdr->SymRecordStream]*g_nBlockSize);
	ListViewInsert(szPos, szData, "SymRecordStream", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, PdbDllRbld), szPos, 8);
	IntToHex(pDbiHdr->PdbDllRbld, szData, 4);
	IntToStr(pDbiHdr->PdbDllRbld, szValue);
	ListViewInsert(szPos, szData, "PdbDllRbld", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, ModInfoSize), szPos, 8);
	IntToHex(pDbiHdr->ModInfoSize, szData, 8);
	IntToStr(pDbiHdr->ModInfoSize, szValue);
	ListViewInsert(szPos, szData, "ModInfoSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, SectionContributionSize), szPos, 8);
	IntToHex(pDbiHdr->SectionContributionSize, szData, 8);
	IntToStr(pDbiHdr->SectionContributionSize, szValue);
	ListViewInsert(szPos, szData, "SectionContributionSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, SectionMapSize), szPos, 8);
	IntToHex(pDbiHdr->SectionMapSize, szData, 8);
	IntToStr(pDbiHdr->SectionMapSize, szValue);
	ListViewInsert(szPos, szData, "SectionMapSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, SourceInfoSize), szPos, 8);
	IntToHex(pDbiHdr->SourceInfoSize, szData, 8);
	IntToStr(pDbiHdr->SourceInfoSize, szValue);
	ListViewInsert(szPos, szData, "SourceInfoSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, TypeServerSize), szPos, 8);
	IntToHex(pDbiHdr->TypeServerSize, szData, 8);
	IntToStr(pDbiHdr->TypeServerSize, szValue);
	ListViewInsert(szPos, szData, "TypeServerSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, MFCTypeServerIndex), szPos, 8);
	IntToHex(pDbiHdr->MFCTypeServerIndex, szData, 8);
	IntToStr(pDbiHdr->MFCTypeServerIndex, szValue);
	ListViewInsert(szPos, szData, "MFCTypeServerIndex", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, OptionalDbgHeaderSize), szPos, 8);
	IntToHex(pDbiHdr->OptionalDbgHeaderSize, szData, 8);
	IntToStr(pDbiHdr->OptionalDbgHeaderSize, szValue);
	ListViewInsert(szPos, szData, "OptionalDbgHeaderSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, ECSubstreamSize), szPos, 8);
	IntToHex(pDbiHdr->ECSubstreamSize, szData, 8);
	IntToStr(pDbiHdr->ECSubstreamSize, szValue);
	ListViewInsert(szPos, szData, "ECSubstreamSize", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, Flags), szPos, 8);
	IntToHex(pDbiHdr->Flags, szData, 4);
	IntToStr(pDbiHdr->Flags, szValue);
	ListViewInsert(szPos, szData, "Flags", szValue);
	IntToHex((DWORD_PTR)pPtr+offsetof(DBISTREAMHEADER, Machine), szPos, 8);
	IntToHex(pDbiHdr->Machine, szData, 4);
	//IntToStr(pDbiHdr->Machine, szValue);
	char *pValue = 0;
	if (pDbiHdr->Machine == 0x014C) pValue = "x86";
	else if (pDbiHdr->Machine == 0x8664) pValue = "x64";
	ListViewInsert(szPos, szData, "Machine", pValue);
}
